import compute from './compute';
import getViewport from './viewport';

var isFunction = function isFunction(arg) {
  return typeof arg === 'function';
};

var isOptionsObject = function isOptionsObject(options) {
  return options === Object(options) && Object.keys(options).length !== 0;
};

var defaultBehavior = function defaultBehavior(actions, behavior) {
  if (behavior === void 0) {
    behavior = 'auto';
  }

  var viewport = getViewport();
  var supportsScrollBehavior = 'scrollBehavior' in viewport.style;
  actions.forEach(function (_ref) {
    var el = _ref.el,
        top = _ref.top,
        left = _ref.left;

    if (el.scroll && supportsScrollBehavior) {
      el.scroll({
        top: top,
        left: left,
        behavior: behavior
      });
    } else {
      if (el === viewport) {
        window.scrollTo(left, top);
      } else {
        el.scrollTop = top;
        el.scrollLeft = left;
      }
    }
  });
};

var getOptions = function getOptions(options) {
  if (options === void 0) {
    options = true;
  }

  if (options === true || options === null) {
    return {
      block: 'start',
      inline: 'nearest'
    };
  } else if (options === false) {
    return {
      block: 'end',
      inline: 'nearest'
    };
  } else if (isOptionsObject(options)) {
    return options;
  }

  return {
    block: 'start',
    inline: 'nearest'
  };
};

function scrollIntoView(target, options) {
  if (options === void 0) {
    options = true;
  }

  if (isOptionsObject(options) && isFunction(options.behavior)) {
    return options.behavior(compute(target, options));
  }

  var computeOptions = getOptions(options);
  return defaultBehavior(compute(target, computeOptions), computeOptions.behavior);
}

export default scrollIntoView;